<?php
/**
 * @category    Pinpoint
 * @package     Pinpoint_Faq
 * @copyright   Copyright (c) 2019 Pinpoint Designs. (http://www.Pinpoint.co.uk)
 * @author      Ahmed Goda <hello@Pinpoint.co.uk>
 */

namespace Pinpoint\Faq\Controller\Product;

use Magento\Backend\App\Action\Context;
use Magento\Framework\App\Action\Action;
use \Magento\Framework\Controller\ResultFactory;
use Magento\Framework\App\ObjectManager;
use Magento\Framework\App\Request\DataPersistorInterface;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\Registry;
use Pinpoint\Faq\Api\FaqRepositoryInterface;
use Pinpoint\Faq\Api\Data\FaqInterface;
use Pinpoint\Faq\Model\Faq;
use Pinpoint\Faq\Model\FaqFactory;
use Pinpoint\Faq\Model\ResourceModel\Faq as FaqResource;
use Pinpoint\Faq\Model\ResourceModel\FaqFactory as FaqResourceFactory;
use Pinpoint\Faq\Model\ResourceModel\Faq\CollectionFactory as CollectionFactory;
use Magento\Store\Model\StoreManagerInterface;

/**
 * Class Post
 */
class Post extends Action
{
    /**
     * @var CollectionFactory
     */
    protected $collectionFactory;
    /**
     * @var DataPersistorInterface
     */
    protected $dataPersistor;

    /**
     * @var FaqFactory
     */
    protected $faqFactory;

    /**
     * @var FaqRepositoryInterface
     */
    protected $faqRepository;

    /**
     * @var FaqResourceFactory
     */
    protected $faqResourceFactory;

    /**
     * @var Registry
     */
    protected $coreRegistry;

    /**
     * @var StoreManagerInterface
     */
    protected $storeManager;

    /**
     * @param Context $context
     * @param Registry $coreRegistry
     * @param DataPersistorInterface $dataPersistor
     * @param FaqResourceFactory|null $faqResourceFactory
     * @param FaqFactory|null $faqFactory
     * @param FaqRepositoryInterface|null $faqRepository
     * @param CollectionFactory $collectionFactory
     * @param StoreManagerInterface $storeManager
     */
    public function __construct(
        Context $context,
        Registry $coreRegistry,
        DataPersistorInterface $dataPersistor,
        FaqResourceFactory $faqResourceFactory = null,
        FaqFactory $faqFactory = null,
        FaqRepositoryInterface $faqRepository = null,
        CollectionFactory $collectionFactory,
        StoreManagerInterface $storeManager
    ) {
        parent::__construct($context);
        $this->coreRegistry       = $coreRegistry;
        $this->dataPersistor      = $dataPersistor;
        $this->faqResourceFactory = $faqResourceFactory ?: ObjectManager::getInstance()->get(FaqResourceFactory::class);
        $this->faqFactory         = $faqFactory ?: ObjectManager::getInstance()->get(FaqFactory::class);
        $this->faqRepository      = $faqRepository ?: ObjectManager::getInstance()->get(FaqRepositoryInterface::class);
        $this->collectionFactory  = $collectionFactory;
        $this->storeManager = $storeManager;
    }

    /**
     * @return \Magento\Framework\Controller\ResultInterface|\Magento\Framework\View\Result\Layout|RedirectResult
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    public function execute()
    {
        /** @var \Magento\Framework\Controller\Result\Redirect $resultRedirect */
        $redirectResult = $this->resultFactory->create(ResultFactory::TYPE_REDIRECT);

        $data = $this->getRequest()->getPostValue();

        $data['store_id'] = $this->storeManager->getStore()->getId();

        if (!$data) {
            return $redirectResult->setPath('*/*/');
        }

        if (empty($data[FaqInterface::FAQ_ID])) {
            $data[FaqInterface::FAQ_ID] = null;
        }

        $faq = $this->createModel();

        $faqId = $this->getRequest()->getParam(FaqInterface::FAQ_ID);

        if ($faqId) {
            try {
                $faq = $this->faqRepository->getById($faqId);
            } catch (LocalizedException $e) {
                $this->messageManager->addErrorMessage(
                    __('This question no longer exists.')
                );
                return $redirectResult->setPath('*/*/');
            }
        }

        $faq->setData($data);

        // Check if it's a duplicate question
        $collection = $this->collectionFactory->create();
        $collection->addFieldToSelect('faq_id')
            ->addFieldToFilter(FaqInterface::QUESTION_TEXT, $data['question_text'])
            ->addFieldToFilter(FaqInterface::PRODUCT_ID, $data['product_id'])
            ->addFieldToFilter(FaqInterface::STORE_ID, $data['store_id'])->load();
        $collection->getColumnValues('faq_id');
        if ($collection->count() > 0) {

            $this->messageManager->addErrorMessage(__('This question already exists for this product.'));

        } else {

            try {
                $this->createResource()->save($faq);
                $this->messageManager->addSuccessMessage(__('Thank you for your question, our team will respond to your query shortly.'));
                $this->dataPersistor->clear('pinpoint_faqs');

            } catch (LocalizedException $e) {
                $this->messageManager->addErrorMessage($e->getMessage());
            } catch (\Exception $e) {
                $this->messageManager->addExceptionMessage($e, __('Something went wrong while saving the FAQ.'));
            }

        }

        $this->dataPersistor->set('pinpoint_faqs', $data);
        $redirectResult->setPath($this->_redirect->getRefererUrl());
        return $redirectResult;
    }

    /**
     * @param array $data
     * @return Faq
     */
    protected function createModel(array $data = []): Faq
    {
        return  $this->faqFactory->create($data);
    }

    /**
     * @return FaqResource
     */
    protected function createResource(): FaqResource
    {
        return $this->faqResourceFactory->create();
    }
}
